@@ -168,10 +168,11 @@ def get_dburi():
         # Assume the schema 'tractest' for PostgreSQL
         if scheme == 'postgres' and \
                 not db_prop.get('params', {}).get('schema'):
-            if '?' in dburi:
-                dburi += "&schema=tractest"
-            else:
-                dburi += "?schema=tractest"
+            dburi += ('&' if '?' in dburi else '?') + 'schema=tractest'
+        elif scheme == 'sqlite' and db_prop['path'] != ':memory:' and \
+                not db_prop.get('params', {}).get('synchronous'):
+            # Speed-up tests with SQLite database
+            dburi += ('&' if '?' in dburi else '?') + 'synchronous=off'
         return dburi
     return 'sqlite::memory:'
 
@@ -331,7 +332,7 @@ class EnvironmentStub(Environment):
             with self.db_transaction as db:
                 db.rollback()  # make sure there's no transaction in progress
                 # check the database version
-                database_version = self.database_version
+                database_version = self.get_version()
         except Exception:
             # "Database not found ...",
             # "OperationalError: no such table: system" or the like
@@ -364,9 +365,6 @@ class EnvironmentStub(Environment):
                 self.global_databasemanager.shutdown()
 
         with self.db_transaction as db:
-            if scheme == 'sqlite':
-                # Speed-up tests with SQLite database
-                db("PRAGMA synchronous = OFF")
             if default_data:
                 for table, cols, vals in db_default.get_data(db):
                     db.executemany("INSERT INTO %s (%s) VALUES (%s)"
