@@ -17,12 +17,12 @@
 
 from __future__ import with_statement
 
-import csv
+from cStringIO import StringIO
+from datetime import datetime, timedelta
 from itertools import groupby
 from math import ceil
-from datetime import datetime, timedelta
+import csv
 import re
-from StringIO import StringIO
 
 from genshi.builder import tag
 
@@ -449,26 +449,36 @@ class Query(object):
         list_fields = [f['name'] for f in self.fields
                                  if f['type'] == 'text' and
                                     f.get('format') == 'list']
+        # 32 is max of joins in SQLite 32-bit, 3 is for order, group and
+        # "priority" columns
+        use_joins = len(set(cols) & set(custom_fields)) <= 32 - 3
 
         sql = []
         sql.append("SELECT " + ",".join(['t.%s AS %s' % (c, c) for c in cols
                                          if c not in custom_fields]))
         sql.append(",priority.value AS priority_value")
-        for k in [db.quote(k) for k in cols if k in custom_fields]:
-            sql.append(",t.%s AS %s" % (k, k))
 
-        # Use subquery of ticket_custom table as necessary
-        if any(k in custom_fields for k in cols):
-            sql.append('\nFROM (\n  SELECT ' +
-                       ','.join('t.%s AS %s' % (c, c)
+        if use_joins:
+            # Use LEFT OUTER JOIN for ticket_custom table
+            sql.extend(",%s.value AS %s" % ((db.quote(k),) * 2)
+                       for k in cols if k in custom_fields)
+            sql.append("\nFROM ticket AS t")
+            sql.extend("\n  LEFT OUTER JOIN ticket_custom AS %(qk)s ON "
+                       "(%(qk)s.ticket=t.id AND %(qk)s.name='%(k)s')"
+                        % {'qk': db.quote(k), 'k': k}
+                        for k in cols if k in custom_fields)
+        else:
+            # Use subquery for ticket_custom table
+            sql.extend(",%s AS %s" % ((db.quote(k),) * 2)
+                       for k in cols if k in custom_fields)
+            sql.append('\nFROM (\n  SELECT ')
+            sql.append(','.join('t.%s AS %s' % (c, c)
                                 for c in cols if c not in custom_fields))
             sql.extend(",\n  (SELECT c.value FROM ticket_custom c "
                        "WHERE c.ticket=t.id AND c.name='%s') AS %s"
                        % (k, db.quote(k))
                        for k in cols if k in custom_fields)
             sql.append("\n  FROM ticket AS t) AS t")
-        else:
-            sql.append("\nFROM ticket AS t")
 
         # Join with the enum table for proper sorting
         for col in [c for c in enum_columns
@@ -494,6 +504,8 @@ class Query(object):
         def get_constraint_sql(name, value, mode, neg):
             if name not in custom_fields:
                 col = 't.' + name
+            elif use_joins:
+                col = db.quote(name) + '.value'
             else:
                 col = 't.' + db.quote(name)
             value = value[len(mode) + neg:]
@@ -596,6 +608,8 @@ class Query(object):
                 elif not mode and len(v) > 1 and k not in self.time_fields:
                     if k not in custom_fields:
                         col = 't.' + k
+                    elif use_joins:
+                        col = db.quote(k) + '.value'
                     else:
                         col = 't.' + db.quote(k)
                     clauses.append("COALESCE(%s,'') %sIN (%s)"
@@ -637,10 +651,12 @@ class Query(object):
         for name, desc in order_cols:
             if name in enum_columns:
                 col = name + '.value'
-            elif name in custom_fields:
-                col = 't.' + db.quote(name)
-            else:
+            elif name not in custom_fields:
                 col = 't.' + name
+            elif use_joins:
+                col = db.quote(name) + '.value'
+            else:
+                col = 't.' + db.quote(name)
             desc = ' DESC' if desc else ''
             # FIXME: This is a somewhat ugly hack.  Can we also have the
             #        column type for this?  If it's an integer, we do first
@@ -865,12 +881,12 @@ class QueryModule(Component):
 
     def convert_content(self, req, mimetype, query, key):
         if key == 'rss':
-            return self.export_rss(req, query)
+            return self._export_rss(req, query)
         elif key == 'csv':
-            return self.export_csv(req, query, mimetype='text/csv')
+            return self._export_csv(req, query, mimetype='text/csv')
         elif key == 'tab':
-            return self.export_csv(req, query, '\t',
-                                   mimetype='text/tab-separated-values')
+            return self._export_csv(req, query, '\t',
+                                    mimetype='text/tab-separated-values')
 
     # INavigationContributor methods
 
@@ -952,9 +968,11 @@ class QueryModule(Component):
         order = args.get('order')
         if isinstance(order, (list, tuple)):
             order = order[0] if order else None
+        group = args.get('group')
+        if isinstance(group, (list, tuple)):
+            group = group[0] if group else None
         query = Query(self.env, report_id,
-                      constraints, cols, order,
-                      'desc' in args, args.get('group'),
+                      constraints, cols, order, 'desc' in args, group,
                       'groupdesc' in args, 'verbose' in args,
                       rows,
                       args.get('page'),
@@ -1152,31 +1170,57 @@ class QueryModule(Component):
         return 'query.html', data, None
 
     def export_csv(self, req, query, sep=',', mimetype='text/plain'):
-        content = StringIO()
-        content.write('\xef\xbb\xbf')   # BOM
-        cols = query.get_columns()
-        writer = csv.writer(content, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
-        writer.writerow([unicode(c).encode('utf-8') for c in cols])
+        """:deprecated: since 1.0.6, use `_export_csv` instead. Will be
+                        removed in 1.3.1.
+        """
+        content, content_type = self._export_csv(req, query, sep, mimetype)
+        return ''.join(content), content_type
 
-        context = web_context(req)
-        results = query.execute(req)
-        for result in results:
-            ticket = Resource('ticket', result['id'])
-            if 'TICKET_VIEW' in req.perm(ticket):
-                values = []
-                for col in cols:
-                    value = result[col]
-                    if col in ('cc', 'owner', 'reporter'):
-                        value = Chrome(self.env).format_emails(
-                                    context.child(ticket), value)
-                    elif col in query.time_fields:
-                        value = format_datetime(value, '%Y-%m-%d %H:%M:%S',
-                                                tzinfo=req.tz)
-                    values.append(unicode(value).encode('utf-8'))
-                writer.writerow(values)
-        return (content.getvalue(), '%s;charset=utf-8' % mimetype)
+    def _export_csv(self, req, query, sep=',', mimetype='text/plain'):
+        def iterate():
+            out = StringIO()
+            writer = csv.writer(out, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
+
+            def writerow(values):
+                writer.writerow([unicode(value).encode('utf-8')
+                                 for value in values])
+                rv = out.getvalue()
+                out.truncate(0)
+                return rv
+
+            yield '\xef\xbb\xbf'  # BOM
+
+            cols = query.get_columns()
+            yield writerow(cols)
+
+            chrome = Chrome(self.env)
+            context = web_context(req)
+            results = query.execute(req)
+            for result in results:
+                ticket = Resource('ticket', result['id'])
+                if 'TICKET_VIEW' in req.perm(ticket):
+                    values = []
+                    for col in cols:
+                        value = result[col]
+                        if col in ('cc', 'owner', 'reporter'):
+                            value = chrome.format_emails(context.child(ticket),
+                                                         value)
+                        elif col in query.time_fields:
+                            value = format_datetime(value, '%Y-%m-%d %H:%M:%S',
+                                                    tzinfo=req.tz)
+                        values.append(value)
+                    yield writerow(values)
+
+        return iterate(), '%s;charset=utf-8' % mimetype
 
     def export_rss(self, req, query):
+        """:deprecated: since 1.0.6, use `_export_rss` instead. Will be
+                        removed in 1.3.1.
+        """
+        content, content_type = self._export_rss(req, query)
+        return ''.join(content), content_type
+
+    def _export_rss(self, req, query):
         context = web_context(req, 'query', absurls=True)
         query_href = query.get_href(context.href)
         if 'description' not in query.rows:
@@ -1188,7 +1232,8 @@ class QueryModule(Component):
             'query_href': query_href
         }
         output = Chrome(self.env).render_template(req, 'query.rss', data,
-                                                  'application/rss+xml')
+                                                  'application/rss+xml',
+                                                  iterable=True)
         return output, 'application/rss+xml'
 
     # IWikiSyntaxProvider methods
