@@ -101,15 +101,17 @@ except ImportError:
 
 def wait_for_file_mtime_change(filename):
     """This function is typically called before a file save operation,
-     waiting if necessary for the file modification time to change. The
-     purpose is to avoid successive file updates going undetected by the
-     caching mechanism that depends on a change in the file modification
-     time to know when the file should be reparsed."""
+    waiting if necessary for the file modification time to change. The
+    purpose is to avoid successive file updates going undetected by the
+    caching mechanism that depends on a change in the file modification
+    time to know when the file should be reparsed."""
+
+    from trac.util import touch_file
     try:
         mtime = os.stat(filename).st_mtime
-        os.utime(filename, None)
+        touch_file(filename)
         while mtime == os.stat(filename).st_mtime:
             time.sleep(1e-3)
-            os.utime(filename, None)
+            touch_file(filename)
     except OSError:
         pass  # file doesn't exist (yet)
