@@ -19,39 +19,45 @@
 import cgi
 import dircache
 import fnmatch
+from functools import partial
 import gc
 import locale
 import os
 import pkg_resources
 from pprint import pformat, pprint
+import re
 import sys
 
-from genshi.core import Markup
-from genshi.builder import Fragment, tag
+from genshi.builder import tag
 from genshi.output import DocType
 from genshi.template import TemplateLoader
 
 from trac import __version__ as TRAC_VERSION
-from trac.config import ExtensionOption, Option, OrderedExtensionsOption
+from trac.config import BoolOption, ExtensionOption, Option, \
+                        OrderedExtensionsOption
 from trac.core import *
 from trac.env import open_environment
 from trac.loader import get_plugin_info, match_plugins_to_frames
 from trac.perm import PermissionCache, PermissionError
 from trac.resource import ResourceNotFound
 from trac.util import arity, get_frame_info, get_last_traceback, hex_entropy, \
-                      read_file, translation
-from trac.util.compat import any, partial
+                      read_file, safe_repr, translation, warn_setuptools_issue
 from trac.util.concurrency import threading
-from trac.util.datefmt import format_datetime, http_date, localtz, timezone
-from trac.util.text import exception_to_unicode, shorten_line, to_unicode
+from trac.util.datefmt import format_datetime, localtz, timezone, user_time
+from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
+                           to_utf8, unicode_quote
 from trac.util.translation import _, get_negotiated_locale, has_babel, \
                                   safefmt, tag_
 from trac.web.api import *
-from trac.web.chrome import Chrome
-from trac.web.clearsilver import HDFWrapper
+from trac.web.chrome import Chrome, add_notice, add_warning
 from trac.web.href import Href
 from trac.web.session import Session
 
+#: This URL is used for semi-automatic bug reports (see
+#: `send_internal_error`).  Please modify it to point to your own
+#: Trac instance if you distribute a patched version of Trac.
+default_tracker = 'http://trac.edgewall.org'
+
 
 class FakeSession(dict):
     sid = None
@@ -59,72 +65,44 @@ class FakeSession(dict):
         pass
 
 
-class FakePerm(dict):
-    def require(self, *args):
-        return False
-    def __call__(self, *args):
+class FakePerm(object):
+
+    username = 'anonymous'
+
+    def __call__(self, realm_or_resource, id=False, version=False):
         return self
 
+    def has_permission(self, action, realm_or_resource=None, id=False,
+                       version=False):
+        return False
 
-def populate_hdf(hdf, env, req=None):
-    """Populate the HDF data set with various information, such as common URLs,
-    project information and request-related information.
-    """
-    # FIXME: do we really have req==None at times?
-    hdf['trac'] = {
-        'version': TRAC_VERSION,
-        'time': format_datetime(),
-        'time.gmt': http_date()
-    }
-    hdf['project'] = {
-        'shortname': os.path.basename(env.path),
-        'name': env.project_name,
-        'name_encoded': env.project_name,
-        'descr': env.project_description,
-        'footer': Markup(env.project_footer),
-        'url': env.project_url
-    }
+    __contains__ = has_permission
 
-    if req:
-        hdf['trac.href'] = {
-            'wiki': req.href.wiki(),
-            'browser': req.href.browser('/'),
-            'timeline': req.href.timeline(),
-            'roadmap': req.href.roadmap(),
-            'milestone': req.href.milestone(None),
-            'report': req.href.report(),
-            'query': req.href.query(),
-            'newticket': req.href.newticket(),
-            'search': req.href.search(),
-            'about': req.href.about(),
-            'about_config': req.href.about('config'),
-            'login': req.href.login(),
-            'logout': req.href.logout(),
-            'settings': req.href.settings(),
-            'homepage': 'http://trac.edgewall.org/'
-        }
+    def require(self, action, realm_or_resource=None, id=False, version=False,
+                message=None):
+        if message is None:
+            raise PermissionError(action)
+        else:
+            raise PermissionError(msg=message)
 
-        hdf['base_url'] = req.base_url
-        hdf['base_host'] = req.base_url[:req.base_url.rfind(req.base_path)]
-        hdf['cgi_location'] = req.base_path
-        hdf['trac.authname'] = req.authname
+    assert_permission = require
 
-        if req.perm:
-            for action in req.perm.permissions():
-                hdf['trac.acl.' + action] = True
 
-        for arg in [k for k in req.args.keys() if k]:
-            if isinstance(req.args[arg], (list, tuple)):
-                hdf['args.%s' % arg] = [v for v in req.args[arg]]
-            elif isinstance(req.args[arg], basestring):
-                hdf['args.%s' % arg] = req.args[arg]
-            # others are file uploads
+class RequestWithSession(Request):
+    """A request that saves its associated session when sending the reply."""
+
+    def send_response(self, code=200):
+        if code < 400:
+            self.session.save()
+        super(RequestWithSession, self).send_response(code)
 
 
 class RequestDispatcher(Component):
     """Web request dispatcher.
-    
-    This component dispatches incoming requests to registered handlers.
+
+    This component dispatches incoming requests to registered
+    handlers.  Besides, it also takes care of user authentication and
+    request pre- and post-processing.
     """
     required = True
 
@@ -138,36 +116,62 @@ class RequestDispatcher(Component):
 
     default_handler = ExtensionOption('trac', 'default_handler',
                                       IRequestHandler, 'WikiModule',
-        """Name of the component that handles requests to the base URL.
-        
-        Options include `TimelineModule`, `RoadmapModule`, `BrowserModule`,
-        `QueryModule`, `ReportModule`, `TicketModule` and `WikiModule`. The
-        default is `WikiModule`. (''since 0.9'')""")
+        """Name of the component that handles requests to the base
+        URL.
+
+        Options include `TimelineModule`, `RoadmapModule`,
+        `BrowserModule`, `QueryModule`, `ReportModule`, `TicketModule`
+        and `WikiModule`. The default is `WikiModule`. (''since 0.9'')""")
 
     default_timezone = Option('trac', 'default_timezone', '',
         """The default timezone to use""")
 
     default_language = Option('trac', 'default_language', '',
-        """The preferred language to use if no user preference has been set.
-        (''since 0.12.1'')
+        """The preferred language to use if no user preference has
+        been set. (''since 0.12.1'')
         """)
 
+    default_date_format = Option('trac', 'default_date_format', '',
+        """The date format. Valid options are 'iso8601' for selecting
+        ISO 8601 format, or leave it empty which means the default
+        date format will be inferred from the browser's default
+        language. (''since 1.0'')
+        """)
+
+    use_xsendfile = BoolOption('trac', 'use_xsendfile', 'false',
+        """When true, send a `X-Sendfile` header and no content when sending
+        files from the filesystem, so that the web server handles the content.
+        This requires a web server that knows how to handle such a header,
+        like Apache with `mod_xsendfile` or lighttpd. (''since 1.0'')
+        """)
+
+    xsendfile_header = Option('trac', 'xsendfile_header', 'X-Sendfile',
+        """The header to use if `use_xsendfile` is enabled. If Nginx is used,
+        set `X-Accel-Redirect`. (''since 1.0.6'')""")
+
     # Public API
 
     def authenticate(self, req):
         for authenticator in self.authenticators:
-            authname = authenticator.authenticate(req)
+            try:
+                authname = authenticator.authenticate(req)
+            except TracError, e:
+                self.log.error("Can't authenticate using %s: %s",
+                               authenticator.__class__.__name__,
+                               exception_to_unicode(e, traceback=True))
+                add_warning(req, _("Authentication error. "
+                                   "Please contact your administrator."))
+                break  # don't fallback to other authenticators
             if authname:
                 return authname
-        else:
-            return 'anonymous'
+        return 'anonymous'
 
     def dispatch(self, req):
-        """Find a registered handler that matches the request and let it process
-        it.
-        
-        In addition, this method initializes the HDF data set and adds the web
-        site chrome.
+        """Find a registered handler that matches the request and let
+        it process it.
+
+        In addition, this method initializes the data dictionary
+        passed to the the template and adds the web site chrome.
         """
         self.log.debug('Dispatching %r', req)
         chrome = Chrome(self.env)
@@ -176,12 +180,14 @@ class RequestDispatcher(Component):
         req.callbacks.update({
             'authname': self.authenticate,
             'chrome': chrome.prepare_request,
-            'hdf': self._get_hdf,
             'perm': self._get_perm,
             'session': self._get_session,
             'locale': self._get_locale,
+            'lc_time': self._get_lc_time,
             'tz': self._get_timezone,
-            'form_token': self._get_form_token
+            'form_token': self._get_form_token,
+            'use_xsendfile': self._get_use_xsendfile,
+            'xsendfile_header': self._get_xsendfile_header,
         })
 
         try:
@@ -198,14 +204,14 @@ class RequestDispatcher(Component):
                             chosen_handler = self.default_handler
                     # pre-process any incoming request, whether a handler
                     # was found or not
-                    chosen_handler = self._pre_process_request(req,
-                                                            chosen_handler)
+                    chosen_handler = \
+                        self._pre_process_request(req, chosen_handler)
                 except TracError, e:
                     raise HTTPInternalError(e)
                 if not chosen_handler:
                     if req.path_info.endswith('/'):
                         # Strip trailing / and redirect
-                        target = req.path_info.rstrip('/').encode('utf-8')
+                        target = unicode_quote(req.path_info.rstrip('/'))
                         if req.query_string:
                             target += '?' + req.query_string
                         req.redirect(req.href + target, permanent=True)
@@ -236,29 +242,26 @@ class RequestDispatcher(Component):
                 # Process the request and render the template
                 resp = chosen_handler.process_request(req)
                 if resp:
-                    if len(resp) == 2: # Clearsilver
-                        chrome.populate_hdf(req)
-                        template, content_type = \
-                                  self._post_process_request(req, *resp)
-                        # Give the session a chance to persist changes
-                        req.session.save()
-                        req.display(template, content_type or 'text/html')
-                    else: # Genshi
-                        template, data, content_type = \
-                                  self._post_process_request(req, *resp)
-                        if 'hdfdump' in req.args:
-                            req.perm.require('TRAC_ADMIN')
-                            # debugging helper - no need to render first
-                            out = StringIO()
-                            pprint(data, out)
-                            req.send(out.getvalue(), 'text/plain')
-                        else:
-                            output = chrome.render_template(req, template,
-                                                            data,
-                                                            content_type)
-                            # Give the session a chance to persist changes
-                            req.session.save()
-                            req.send(output, content_type or 'text/html')
+                    if len(resp) == 2: # old Clearsilver template and HDF data
+                        self.log.error("Clearsilver template are no longer "
+                                       "supported (%s)", resp[0])
+                        raise TracError(
+                            _("Clearsilver templates are no longer supported, "
+                              "please contact your Trac administrator."))
+                    # Genshi
+                    template, data, content_type = \
+                              self._post_process_request(req, *resp)
+                    if 'hdfdump' in req.args:
+                        req.perm.require('TRAC_ADMIN')
+                        # debugging helper - no need to render first
+                        out = StringIO()
+                        pprint(data, out)
+                        req.send(out.getvalue(), 'text/plain')
+
+                    output = chrome.render_template(
+                            req, template, data, content_type,
+                            iterable=chrome.use_chunked_encoding)
+                    req.send(output, content_type or 'text/html')
                 else:
                     self._post_process_request(req)
             except RequestDone:
@@ -276,7 +279,7 @@ class RequestDispatcher(Component):
                                    exception_to_unicode(e, traceback=True))
                 raise err[0], err[1], err[2]
         except PermissionError, e:
-            raise HTTPForbidden(to_unicode(e))
+            raise HTTPForbidden(e)
         except ResourceNotFound, e:
             raise HTTPNotFound(e)
         except TracError, e:
@@ -284,16 +287,11 @@ class RequestDispatcher(Component):
 
     # Internal methods
 
-    def _get_hdf(self, req):
-        hdf = HDFWrapper(loadpaths=Chrome(self.env).get_all_templates_dirs())
-        populate_hdf(hdf, self.env, req)
-        return hdf
-
     def _get_perm(self, req):
         if isinstance(req.session, FakeSession):
             return FakePerm()
         else:
-            return PermissionCache(self.env, self.authenticate(req))
+            return PermissionCache(self.env, req.authname)
 
     def _get_session(self, req):
         try:
@@ -309,36 +307,65 @@ class RequestDispatcher(Component):
             default = self.env.config.get('trac', 'default_language', '')
             negotiated = get_negotiated_locale([preferred, default] +
                                                req.languages)
-            self.log.debug("Negotiated locale: %s -> %s", preferred, negotiated)
+            self.log.debug("Negotiated locale: %s -> %s", preferred,
+                           negotiated)
             return negotiated
 
+    def _get_lc_time(self, req):
+        lc_time = req.session.get('lc_time')
+        if not lc_time or lc_time == 'locale' and not has_babel:
+            lc_time = self.default_date_format
+        if lc_time == 'iso8601':
+            return 'iso8601'
+        return req.locale
+
     def _get_timezone(self, req):
         try:
             return timezone(req.session.get('tz', self.default_timezone
                                             or 'missing'))
-        except:
+        except Exception:
             return localtz
 
     def _get_form_token(self, req):
         """Used to protect against CSRF.
 
-        The 'form_token' is strong shared secret stored in a user cookie.
-        By requiring that every POST form to contain this value we're able to
-        protect against CSRF attacks. Since this value is only known by the
-        user and not by an attacker.
-        
+        The 'form_token' is strong shared secret stored in a user
+        cookie.  By requiring that every POST form to contain this
+        value we're able to protect against CSRF attacks. Since this
+        value is only known by the user and not by an attacker.
+
         If the the user does not have a `trac_form_token` cookie a new
         one is generated.
         """
-        if req.incookie.has_key('trac_form_token'):
+        if 'trac_form_token' in req.incookie:
             return req.incookie['trac_form_token'].value
         else:
             req.outcookie['trac_form_token'] = hex_entropy(24)
             req.outcookie['trac_form_token']['path'] = req.base_path or '/'
             if self.env.secure_cookies:
                 req.outcookie['trac_form_token']['secure'] = True
+            if sys.version_info >= (2, 6):
+                req.outcookie['trac_form_token']['httponly'] = True
             return req.outcookie['trac_form_token'].value
 
+    def _get_use_xsendfile(self, req):
+        return self.use_xsendfile
+
+    # RFC7230 3.2 Header Fields
+    _xsendfile_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
+    _warn_xsendfile_header = False
+
+    def _get_xsendfile_header(self, req):
+        header = self.xsendfile_header.strip()
+        if self._xsendfile_header_re.match(header):
+            return to_utf8(header)
+        else:
+            if not self._warn_xsendfile_header:
+                self._warn_xsendfile_header = True
+                self.log.warn("[trac] xsendfile_header is invalid: '%s'",
+                              header)
+            return None
+
     def _pre_process_request(self, req, chosen_handler):
         for filter_ in self.filters:
             chosen_handler = filter_.pre_process_request(req, chosen_handler)
@@ -348,7 +375,7 @@ class RequestDispatcher(Component):
         nbargs = len(args)
         resp = args
         for f in reversed(self.filters):
-            # As the arity of `post_process_request` has changed since 
+            # As the arity of `post_process_request` has changed since
             # Trac 0.10, only filters with same arity gets passed real values.
             # Errors will call all filters with None arguments,
             # and results will not be not saved.
@@ -360,13 +387,21 @@ class RequestDispatcher(Component):
         return resp
 
 
+_warn_setuptools = False
+_slashes_re = re.compile(r'/+')
+
 def dispatch_request(environ, start_response):
     """Main entry point for the Trac web interface.
-    
-    @param environ: the WSGI environment dict
-    @param start_response: the WSGI callback for starting the response
+
+    :param environ: the WSGI environment dict
+    :param start_response: the WSGI callback for starting the response
     """
 
+    global _warn_setuptools
+    if _warn_setuptools is False:
+        _warn_setuptools = True
+        warn_setuptools_issue(out=environ.get('wsgi.errors'))
+
     # SCRIPT_URL is an Apache var containing the URL before URL rewriting
     # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
     script_url = environ.get('SCRIPT_URL')
@@ -374,8 +409,12 @@ def dispatch_request(environ, start_response):
         path_info = environ.get('PATH_INFO')
         if not path_info:
             environ['SCRIPT_NAME'] = script_url
-        elif script_url.endswith(path_info):
-            environ['SCRIPT_NAME'] = script_url[:-len(path_info)]
+        else:
+            # mod_wsgi squashes slashes in PATH_INFO (!)
+            script_url = _slashes_re.sub('/', script_url)
+            path_info = _slashes_re.sub('/', path_info)
+            if script_url.endswith(path_info):
+                environ['SCRIPT_NAME'] = script_url[:-len(path_info)]
 
     # If the expected configuration keys aren't found in the WSGI environment,
     # try looking them up in the process environment variables
@@ -389,7 +428,7 @@ def dispatch_request(environ, start_response):
     environ.setdefault('trac.locale', '')
     environ.setdefault('trac.base_url',
                        os.getenv('TRAC_BASE_URL'))
-    
+
 
     locale.setlocale(locale.LC_ALL, environ['trac.locale'])
 
@@ -412,7 +451,7 @@ def dispatch_request(environ, start_response):
                 return []
 
             errmsg = None
-            
+
             # To make the matching patterns of request handlers work, we append
             # the environment name to the `SCRIPT_NAME` variable, and keep only
             # the remaining path in the `PATH_INFO` variable.
@@ -434,7 +473,7 @@ def dispatch_request(environ, start_response):
                 errmsg = 'Invalid URL encoding (was %r)' % script_name
 
             if errmsg:
-                start_response('404 Not Found', 
+                start_response('404 Not Found',
                                [('Content-Type', 'text/plain'),
                                 ('Content-Length', str(len(errmsg)))])
                 return [errmsg]
@@ -458,23 +497,23 @@ def dispatch_request(environ, start_response):
             mod_wsgi_version = environ.get('mod_wsgi.version')
             if mod_wsgi_version:
                 mod_wsgi_version = (
-                        "%s (WSGIProcessGroup %s WSGIApplicationGroup %s)" % 
+                        "%s (WSGIProcessGroup %s WSGIApplicationGroup %s)" %
                         ('.'.join([str(x) for x in mod_wsgi_version]),
                          environ.get('mod_wsgi.process_group'),
-                         environ.get('mod_wsgi.application_group') or 
+                         environ.get('mod_wsgi.application_group') or
                          '%{GLOBAL}'))
                 environ.update({
                     'trac.web.frontend': 'mod_wsgi',
                     'trac.web.version': mod_wsgi_version})
             env.webfrontend = environ.get('trac.web.frontend')
             if env.webfrontend:
-                env.systeminfo.append((env.webfrontend, 
+                env.systeminfo.append((env.webfrontend,
                                        environ['trac.web.version']))
     except Exception, e:
         env_error = e
 
-    req = Request(environ, start_response)
-    translation.make_activable(lambda: req.locale, env and env.path or None)
+    req = RequestWithSession(environ, start_response)
+    translation.make_activable(lambda: req.locale, env.path if env else None)
     try:
         return _dispatch_request(req, env, env_error)
     finally:
@@ -501,7 +540,7 @@ def _dispatch_request(req, env, env_error):
 
     # fixup env.abs_href if `[trac] base_url` was not specified
     if env and not env.abs_href.base:
-        env._abs_href = req.abs_href
+        env.abs_href = req.abs_href
 
     try:
         if not env and env_error:
@@ -509,69 +548,50 @@ def _dispatch_request(req, env, env_error):
         try:
             dispatcher = RequestDispatcher(env)
             dispatcher.dispatch(req)
-        except RequestDone:
-            pass
-        resp = req._response or []
-
+        except RequestDone, req_done:
+            resp = req_done.iterable
+        resp = resp or req._response or []
     except HTTPException, e:
-        # This part is a bit more complex than it should be.
-        # See trac/web/api.py for the definition of HTTPException subclasses.
-        if env:
-            env.log.warn(exception_to_unicode(e))
-        try:
-            # We try to get localized error messages here, 
-            # but we should ignore secondary errors
-            title = _('Error')
-            if e.reason:
-                if title.lower() in e.reason.lower():
-                    title = e.reason
-                else:
-                    title = _('Error: %(message)s', message=e.reason)
-        except:
-            title = 'Error'
-        # The message is based on the e.detail, which can be an Exception
-        # object, but not a TracError one: when creating HTTPException,
-        # a TracError.message is directly assigned to e.detail
-        if isinstance(e.detail, Exception): # not a TracError
-            message = exception_to_unicode(e.detail)
-        elif isinstance(e.detail, Fragment): # markup coming from a TracError
-            message = e.detail
-        else:
-            message = to_unicode(e.detail)
-        data = {'title': title, 'type': 'TracError', 'message': message,
-                'frames': [], 'traceback': None}
-        if e.code == 403 and req.authname == 'anonymous':
-            # TRANSLATOR: ... not logged in, you may want to 'do so' now (link)
-            do_so = tag.a(_("do so"), href=req.href.login())
-            req.chrome['notices'].append(
-                tag_("You are currently not logged in. You may want to "
-                     "%(do_so)s now.", do_so=do_so))
-        try:
-            req.send_error(sys.exc_info(), status=e.code, env=env, data=data)
-        except RequestDone:
-            pass
-
-    except Exception, e:
+        _send_user_error(req, env, e)
+    except Exception:
         send_internal_error(env, req, sys.exc_info())
-
     return resp
 
 
+def _send_user_error(req, env, e):
+    # See trac/web/api.py for the definition of HTTPException subclasses.
+    if env:
+        env.log.warn('[%s] %s', req.remote_addr, exception_to_unicode(e))
+    data = {'title': e.title, 'type': 'TracError', 'message': e.message,
+            'frames': [], 'traceback': None}
+    if e.code == 403 and req.authname == 'anonymous':
+        # TRANSLATOR: ... not logged in, you may want to 'do so' now (link)
+        do_so = tag.a(_("do so"), href=req.href.login())
+        add_notice(req, tag_("You are currently not logged in. You may want "
+                             "to %(do_so)s now.", do_so=do_so))
+    try:
+        req.send_error(sys.exc_info(), status=e.code, env=env, data=data)
+    except RequestDone:
+        pass
+
+
 def send_internal_error(env, req, exc_info):
     if env:
-        env.log.error("Internal Server Error: %s", 
+        env.log.error("Internal Server Error: %s",
                       exception_to_unicode(exc_info[1], traceback=True))
     message = exception_to_unicode(exc_info[1])
     traceback = get_last_traceback()
 
     frames, plugins, faulty_plugins = [], [], []
-    tracker = 'http://trac.edgewall.org'
     th = 'http://trac-hacks.org'
     has_admin = False
     try:
         has_admin = 'TRAC_ADMIN' in req.perm
     except Exception:
         pass
+
+    tracker = default_tracker
+    tracker_args = {}
     if has_admin and not isinstance(exc_info[1], MemoryError):
         # Collect frame and plugin information
         frames = get_frame_info(exc_info[2])
@@ -581,7 +601,7 @@ def send_internal_error(env, req, exc_info):
                               for m in p['modules'].itervalues()
                               for c in m['components'].itervalues())]
             match_plugins_to_frames(plugins, frames)
-        
+
             # Identify the tracker where the bug should be reported
             faulty_plugins = [p for p in plugins if 'frame_idx' in p]
             faulty_plugins.sort(key=lambda p: p['frame_idx'])
@@ -591,13 +611,18 @@ def send_internal_error(env, req, exc_info):
                     tracker = info['trac']
                 elif info.get('home_page', '').startswith(th):
                     tracker = th
+                    plugin_name = info.get('home_page', '').rstrip('/') \
+                                                           .split('/')[-1]
+                    tracker_args = {'component': plugin_name}
 
     def get_description(_):
         if env and has_admin:
             sys_info = "".join("|| '''`%s`''' || `%s` ||\n"
                                % (k, v.replace('\n', '` [[br]] `'))
                                for k, v in env.get_systeminfo())
-            sys_info += "|| '''`jQuery`''' || `#JQUERY#` ||\n"
+            sys_info += "|| '''`jQuery`''' || `#JQUERY#` ||\n" \
+                        "|| '''`jQuery UI`''' || `#JQUERYUI#` ||\n" \
+                        "|| '''`jQuery Timepicker`''' || `#JQUERYTP#` ||\n"
             enabled_plugins = "".join("|| '''`%s`''' || `%s` ||\n"
                                       % (p['name'], p['version'] or _('N/A'))
                                       for p in plugins)
@@ -639,40 +664,37 @@ User agent: `#USER_AGENT#`
     data = {'title': 'Internal Error',
             'type': 'internal', 'message': message,
             'traceback': traceback, 'frames': frames,
-            'shorten_line': shorten_line,
+            'shorten_line': shorten_line, 'repr': safe_repr,
             'plugins': plugins, 'faulty_plugins': faulty_plugins,
-            'tracker': tracker,
+            'tracker': tracker, 'tracker_args': tracker_args,
             'description': description, 'description_en': description_en}
 
+    Chrome(env).add_jquery_ui(req)
     try:
         req.send_error(exc_info, status=500, env=env, data=data)
     except RequestDone:
         pass
 
-    
+
 def send_project_index(environ, start_response, parent_dir=None,
                        env_paths=None):
     req = Request(environ, start_response)
 
     loadpaths = [pkg_resources.resource_filename('trac', 'templates')]
-    use_clearsilver = False
     if req.environ.get('trac.env_index_template'):
-        tmpl_path, template = os.path.split(req.environ['trac.env_index_template'])
+        env_index_template = req.environ['trac.env_index_template']
+        tmpl_path, template = os.path.split(env_index_template)
         loadpaths.insert(0, tmpl_path)
-        use_clearsilver = template.endswith('.cs') # assume Clearsilver
-        if use_clearsilver:
-            req.hdf = HDFWrapper(loadpaths) # keep that for custom .cs templates
     else:
         template = 'index.html'
 
-    data = {'trac': {'version': TRAC_VERSION, 'time': format_datetime()},
+    data = {'trac': {'version': TRAC_VERSION,
+                     'time': user_time(req, format_datetime)},
             'req': req}
     if req.environ.get('trac.template_vars'):
         for pair in req.environ['trac.template_vars'].split(','):
             key, val = pair.split('=')
             data[key] = val
-            if use_clearsilver:
-                req.hdf[key] = val
     try:
         href = Href(req.base_path)
         projects = []
@@ -692,23 +714,26 @@ def send_project_index(environ, start_response, parent_dir=None,
         projects.sort(lambda x, y: cmp(x['name'].lower(), y['name'].lower()))
 
         data['projects'] = projects
-        if use_clearsilver:
-            req.hdf['projects'] = projects
-            req.display(template)
 
-        loader = TemplateLoader(loadpaths, variable_lookup='lenient')
+        loader = TemplateLoader(loadpaths, variable_lookup='lenient',
+                                default_encoding='utf-8')
         tmpl = loader.load(template)
         stream = tmpl.generate(**data)
-        output = stream.render('xhtml', doctype=DocType.XHTML_STRICT)
-        req.send(output, 'text/html')
+        if template.endswith('.xml'):
+            output = stream.render('xml')
+            req.send(output, 'text/xml')
+        else:
+            output = stream.render('xhtml', doctype=DocType.XHTML_STRICT,
+                                   encoding='utf-8')
+            req.send(output, 'text/html')
 
     except RequestDone:
         pass
 
 
 def get_tracignore_patterns(env_parent_dir):
-    """Return the list of patterns from env_parent_dir/.tracignore or a
-    default pattern of `".*"` if the file doesn't exist.
+    """Return the list of patterns from env_parent_dir/.tracignore or
+    a default pattern of `".*"` if the file doesn't exist.
     """
     path = os.path.join(env_parent_dir, '.tracignore')
     try:
@@ -721,8 +746,8 @@ def get_tracignore_patterns(env_parent_dir):
 def get_environments(environ, warn=False):
     """Retrieve canonical environment name to path mapping.
 
-    The environments may not be all valid environments, but they are good
-    candidates.
+    The environments may not be all valid environments, but they are
+    good candidates.
     """
     env_paths = environ.get('trac.env_paths', [])
     env_parent_dir = environ.get('trac.env_parent_dir')
@@ -736,7 +761,7 @@ def get_environments(environ, warn=False):
         paths = [path[:-1] for path in paths if path[-1] == '/'
                  and not any(fnmatch.fnmatch(path[:-1], pattern)
                              for pattern in ignore_patterns)]
-        env_paths.extend(os.path.join(env_parent_dir, project) \
+        env_paths.extend(os.path.join(env_parent_dir, project)
                          for project in paths)
     envs = {}
     for env_path in env_paths:
